<?php
namespace metagen_php;
use Exception;

function array_extract_val(&$arr, $assoc, $name)
{
  if(!is_array($arr))
    throw new Exception("$name: Not an array");

  if(!$assoc)
  {
    if(sizeof($arr) == 0)
    {
      if(func_num_args() == 4)
        return func_get_arg(3);
      throw new Exception("$name: No next array item");
    }
    return array_shift($arr);
  }

  if(!array_key_exists($name, $arr))
  {
    if(func_num_args() == 4)
      return func_get_arg(3);
    throw new Exception("$name: No array item");
  }

  $val = $arr[$name];
  unset($arr[$name]);
  return $val;
}

function array_set_value(&$arr, $assoc, $name, $value)
{
  if($assoc)
    $arr[$name] = $value;
  else
    $arr[] = $value;
}

function val_string($val)
{
  //special case for empty strings
  if(is_bool($val) && $val === false)
    return '';
  if(!is_string($val))
    throw new Exception("Bad item, not a string(" . serialize($val) . ")");
  return $val;
}

function val_bool($val)
{
  if(!is_bool($val))
    throw new Exception("Bad item, not a bool(" . serialize($val) . ")");
  return $val;
}

function val_float($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  return $val;
}

function val_double($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  return 1*$val;
}

function val_uint64($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected uint64, float given: $val");
  return $val;
}

function val_int64($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected int64, float given: $val");
  return $val;
}

function val_uint32($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected uint32, float given: $val");
  if(($val < 0 && $val < -2147483648) || ($val > 0 && $val > 0xFFFFFFFF))
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_int32($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected int32, float given: $val");
  if($val > 2147483647 || $val < -2147483648)
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_uint16($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected uint16, float given: $val");
  if($val > 0xFFFF || $val < 0)
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_int16($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected int16, float given: $val");
  if($val > 32767 || $val < -32768)
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_uint8($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected uint8, float given: $val");
  if($val > 0xFF || $val < 0)
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_int8($val)
{
  if(!is_numeric($val))
    throw new Exception("Bad item, not a number(" . serialize($val) . ")");
  $val = 1*$val;
  if(is_float($val))
    throw new Exception("Incompatible type, expected int8, float given: $val");
  if($val > 127 || $val < -128)
    throw new Exception("Value not in range: $val");
  return $val;
}

function val_arr($val)
{
  if(!is_array($val))
    throw new Exception("Bad item, not an array(" . serialize($val) . ")");
  return $val;
}

function val_enum($enum, $val, $check_enum_validity = true)
{
  if(is_string($val))
    return call_user_func_array(array($enum, "getValueByName"), array($val));

  if(!is_numeric($val))
    throw new Exception("Bad enum value, not a numeric or string(" . serialize($val) . ")");

  if($check_enum_validity)
    call_user_func_array(array($enum, "checkValidity"), array($val));
  return $val;
}

function val_blob($val)
{
  if(is_null($val))
    return $val;

  if(!is_string($val))
    throw new Exception("Bad item, not a blob string(" . serialize($val) . ")");

  return $val;
}
